#include "mbed.h"
#include "AM2303.h"
#include "Luminosite.h"
#include "gas.h"
#include "Akene.h"
#include "Dust.h"
#include "Air_Quality.h"

/*** Humidity & Temperature sensor **/
AM2303 h(D6);
int state_h;

/** comminication Serie **/
Serial serial (USBTX, USBRX);

/** Capteur Luminisoté **/
AnalogIn luminosite(A0);

/** Alcohol **/
AnalogIn alcohol(A2);

/* Dust sensor */
DigitalOut dustDig(D8);
AnalogIn dustAnal(A3); 

/* AirQuality */

AirQuality airqualitysensor;
int current_quality = -1;
PinName analogPin = A1;

void AirQualityInterrupt()
{
    AnalogIn sensor(analogPin);
    airqualitysensor.last_vol = airqualitysensor.first_vol;
    airqualitysensor.first_vol = sensor.read()*1000;
    airqualitysensor.timer_index = 1;
}

/*********************Sigfox*********************/


struct  capteur_data{
    short temp;
    short hum;
    short lum;//1 octet
    short gas;
    short Dust; // short
    short airQuality;
};

capteur_data data;

void sendMessage (capteur_data test) {
    Akene_ akene;
    akene.begin();
    //int test = 1;
    if(!akene.isReady()){
              serial.printf("IS NOT READY");
        wait(1);     
    }
    akene.send(&test, sizeof(test)); 
}

/********************* MAIN *********************/

void init () {
    serial.baud(9600);
    airqualitysensor.init(analogPin, AirQualityInterrupt);
}

void loop () {
    wait(3.0);
    int state_h = h.readData();
    if (state_h != AM2303::OK) {
        serial.printf("<Error: %d>\n", state_h);
    } 
    else {
        data.temp = h.readTemperature();
        data.hum = h.readHumidity();
        serial.printf("T:%hu C,  H:%hu \n",data.temp,data.hum);
    }
    data.lum = computeLuminosite(luminosite.read());
    data.gas = alcohol.read()*100;
    data.Dust = computeDust(&dustAnal,&dustDig)*100;
    data.airQuality=airqualitysensor.slope();
    serial.printf("LUM: %f  \n",luminosite.read());
    serial.printf("LUM: %hu  \n",data.lum);
    serial.printf("GAS: %hu  \n",data.gas);
    serial.printf("DUST : %hu  ",data.Dust);
    serial.printf("Air Q: %hu \n", data.airQuality);
    serial.printf("****************************************************\n\n\n");
    sendMessage(data);
}

int main() {
    init ();
    while (1) loop();
}